#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
 
#include <linux/i2c-dev.h>
#include <fcntl.h>
#include <sys/ioctl.h>
 
#define LCD_CONTRAST 100
 
#define LCD_RS_CMD (0x00)
#define LCD_RS_DATA (0x40)
#define LCD_CMD_CLEAR (0x01)
#define LCD_CMD_HOME (0x03)

int lcd_Address = 0x3E;  // I2C LCD のアドレス。
int LPS331AP_Address = 0x5D; //LPS331APのアドレス。
 
// 関数プロトタイプ宣言。
void LCD_init(int fd);
void LCD_write(unsigned char rs, unsigned char data, int fd);
void LCD_clear(int fd);
void LCD_setCursor(unsigned char col, unsigned char row, int fd);
void LCD_putc(unsigned char c, int fd);
void LCD_puts(char *str, int fd);

double LPS331AP_readData(int fd);
unsigned char LPS331AP_read(unsigned char address, int fd);
void LPS331AP_write(unsigned char address, unsigned char data, int fd);
void LPS331AP_init(int fd);

void LCD_icons(int i,int lcd_fd);

int main(int argc, char **argv){
    int lcd_fd, lps331ap_fd;       // ファイルディスクリプタ。
    char *i2cFileName = "/dev/i2c-1"; // I2Cドライバファイル名。
    char buf[16];
    int i = 0;

    //LCD用デバイスファイルをオープン
    if ((lcd_fd = open(i2cFileName, O_RDWR)) < 0){
        printf("Faild to open i2c port\n");
        exit(1);
    }
 
    // 通信先アドレスの設定。
    if (ioctl(lcd_fd, I2C_SLAVE, lcd_Address) < 0){
        printf("Unable to access to LCD device\n");
        exit(1);
    }
    
    // LPS331AP用デバイスファイルをオープン
    if ((lps331ap_fd = open(i2cFileName, O_RDWR)) < 0) 
    {
        printf("Faild to open i2c port\n");
        exit(1);
    }

    // LPS331AP用にセット
    if (ioctl(lps331ap_fd, I2C_SLAVE, LPS331AP_Address) < 0) 
    {
        printf("Unable to access to LPS331AP\n");
        exit(1);
    }
 
    //デバイス初期化
    LPS331AP_init(lps331ap_fd);

    // LCD初期化。
    LCD_init(lcd_fd);

    for(i = 0; i < 17; i++){
        LCD_icons(i,lcd_fd);
    }

   for(i = 0; i < 200; i++){
        // メッセージ表示。
        LCD_setCursor(0, 0, lcd_fd);
        LCD_puts("pressure", lcd_fd);
        LCD_setCursor(0, 1, lcd_fd);
        sprintf(buf,"%f",LPS331AP_readData(lps331ap_fd));
        LCD_puts(buf, lcd_fd);
        usleep(1000000);
    }
   return 0;

}
 
void LCD_init(int fd){
 
    // ST7032データシート の初期化サンプルの通り
    printf("initializing LPS331AP \n");
 
    // Function Set : 8bit bus mode, 2-line mode,normal font,normal instruction mode
    LCD_write(LCD_RS_CMD, 0b00111000, fd);      // 0x38
    // Function Set : extension instruction mode
    LCD_write(LCD_RS_CMD, 0b00111001, fd);      // 0x39
    // Internal OSC frequency(extension instruction mode)
    LCD_write(LCD_RS_CMD, 0b00010100, fd);      // 0x14
    // Contrast set(extension instruction mode) コントラスト値下位4bit設定
    LCD_write(LCD_RS_CMD, 0b01110000 | (LCD_CONTRAST & 0xF), fd);   // 0x78 = 0x70 + 0x8
    // Power/ICON/Contrast set(extension instruction mode) コントラスト値上位2bit設定
    LCD_write(LCD_RS_CMD, 0b01011100 | ((LCD_CONTRAST >> 4) & 0x3), fd);    // 0x5c + 0
    // Follower control。internal follower on, 
    LCD_write(LCD_RS_CMD, 0b01101100, fd);      // 0x6c
 
    usleep(300000);   // 300ミリ秒
 
    // Function Set。normal instruction mode。
    LCD_write(LCD_RS_CMD, 0b00111000, fd);    // 0x38
    // Display On
    LCD_write(LCD_RS_CMD, 0b00001100, fd);      // 0x0c
    // Clear Display
    LCD_write(LCD_RS_CMD, 0b00000001, fd);      // 0x01
    // Entry Mode Set
    LCD_write(LCD_RS_CMD, 0b00000110, fd);      // 0x06
    // 時間待ち。
    usleep(2000);

    printf("initialization complete\n");
 

}
 
void LCD_write(unsigned char rs, unsigned char data, int fd)
{
    unsigned char buf[2];
 
    if (rs == LCD_RS_CMD || rs == LCD_RS_DATA){
        // LCD_RS_CMD ならコマンドモード。LCD_RS_DATA ならデータモード。
 
        buf[0] = rs;
        buf[1] = data;
        if (write(fd, buf, 2) != 2){
            printf("Error writing to i2c slave\n");
        }        
    }
    else{
        // rsの指定がLCD_RS_CMD,LCD_RS_DATA以外ならなにもしない。
    }
}
 
void LCD_clear(int fd)
{
    LCD_write(LCD_RS_CMD, LCD_CMD_CLEAR, fd);
    usleep(2000);
    LCD_write(LCD_RS_CMD, LCD_CMD_HOME, fd);
    usleep(2000);
}
 
void LCD_setCursor(unsigned char col, unsigned char row, int fd)
{
    unsigned char offset[] = {0x00, 0x40};
 
    if (row > 1) row = 1;
    if (col > 16) col = 16;
    
    LCD_write(LCD_RS_CMD, 0x80 | (col + offset[row]), fd);
}
 
void LCD_putc(unsigned char c, int fd)
{
    LCD_write(LCD_RS_DATA, c, fd);
}
 
void LCD_puts(char *str, int fd)
{
    int i;
    for (i = 0; i < 16; i++){
        if (str[i] == 0x00){
            break;
        }
        else{
            LCD_putc((unsigned int)str[i], fd);
        }
    }
}
 
// LPS331AP用 1バイト読み出しルーチン： addressで示すレジスタの値を読み出す
// 戻り値がレジスタ値
unsigned char LPS331AP_read(unsigned char address, int fd)
{
    unsigned char buf[1];
    buf[0] = address;
    if((write(fd,buf,1))!= 1){ // addressを一度書き込む所に注意
        printf("Error writing to i2c slave\n");
        exit(1);
    }
    if(read(fd,buf,1)!=1){
        printf("Error reading from i2c slave\n");
        exit(1);
    }
    return buf[0];
}

// LPS331AP用 1バイト書き込みルーチン：addressで示すレジスタにdataを書き込む
void LPS331AP_write(unsigned char address, unsigned char data, int fd)
{
    unsigned char buf[2];
    buf[0] = address;
    buf[1] = data;
    if((write(fd,buf,2))!=2){
        printf("Error writing to i2c slave\n");
        exit(1);
    }
    return;
}

// LPS331AP用 大気圧読み出しルーチン：
// 整数値配列へのポインタを使ってデータを受け渡す
double LPS331AP_readData(int fd)
{
    unsigned char data[3];
    int tmpdata;
    double tmpdata1;

    int i;

    for(i = 0; i < 3; i++){
        data[i] = LPS331AP_read(0x28 + i,fd);
    }
    
    tmpdata = (data[2] << 16) | (data[1] << 8) | data[0];
    tmpdata1 = (float)tmpdata/4096;
    printf("pressure = %f\n",tmpdata1);

   return tmpdata1;
}


 // LPS331AP用 大気圧センサイニシャライズルーチン
void LPS331AP_init(int fd)
{
    unsigned char Data;

    int tmpdata;
    double tmpdata1;

    printf("checking the device....\n");
    Data = LPS331AP_read(0x0f,fd);

    if( Data != 0xBB){
        printf("there are no LPS331AP\n");
        exit(1);
    }

    printf("initializing LPS331AP \n");
    LPS331AP_write(0x20, 0x90, fd);

    Data = LPS331AP_read(0x20,fd);

    if(Data != 0x90){
        printf("failed initialization\n");
        exit(1);
    }
    
    printf("initialization complete\n");

    usleep(10);


    return;
}

/*アイコンの表示*/
void LCD_icons(int i,int lcd_fd){

    /* set instruction table (IS = 1) */
    usleep(100000);
    LCD_write(LCD_RS_CMD, 0b00111001, lcd_fd);    // 0x39

    if(i == 0){
        LCD_write(LCD_RS_CMD, 0x40,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else if(i == 1){
        LCD_write(LCD_RS_CMD, 0x42,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else if(i == 2){
        LCD_write(LCD_RS_CMD, 0x44,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else if(i == 3){
        LCD_write(LCD_RS_CMD, 0x46,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else if(i == 4){
        LCD_write(LCD_RS_CMD, 0x47,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else if(i == 5){
        LCD_write(LCD_RS_CMD, 0x47,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00001000,lcd_fd);
    }else if(i == 6){
        LCD_write(LCD_RS_CMD, 0x47,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00011000,lcd_fd);
    }else if(i == 7){
        LCD_write(LCD_RS_CMD, 0x49,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else if(i == 8){
        LCD_write(LCD_RS_CMD, 0x4B,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else if(i == 9){
        LCD_write(LCD_RS_CMD, 0x4D,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else if(i == 10){
        LCD_write(LCD_RS_CMD, 0x4D,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00001000,lcd_fd);
    }else if(i == 11){
        LCD_write(LCD_RS_CMD, 0x4D,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00000100,lcd_fd);
    }else if(i == 12){
        LCD_write(LCD_RS_CMD, 0x4D,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00000010,lcd_fd);
    }else if(i == 13){
        LCD_write(LCD_RS_CMD, 0x4D,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010010,lcd_fd);
    }else if(i == 14){
        LCD_write(LCD_RS_CMD, 0x4D,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00011010,lcd_fd);
    }else if(i == 15){
        LCD_write(LCD_RS_CMD, 0x4D,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00011110,lcd_fd);
    }else if(i == 16){
        LCD_write(LCD_RS_CMD, 0x4F,lcd_fd);
        LCD_write(LCD_RS_DATA, 0b00010000,lcd_fd);
    }else{
      printf("No such Icon\n");
    }
    
    /* Reset Instruction table to IS = 0 */
    LCD_write(LCD_RS_CMD, 0b00111000, lcd_fd);    // 0x38

}
 
